"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const find_up_1 = __importDefault(require("find-up"));
const promises_1 = __importDefault(require("fs/promises"));
const log_1 = __importDefault(require("./helpers/log"));
const FirebaseConfigFileName = 'firebase.json';
module.exports =
    (sites) => (nextConfig = {}) => {
        if (!sites || sites.length < 1) {
            throw new Error(`In order for this to work provide a list of sites to check for rewrites in the firebase.json hosting section.`);
        }
        return Object.assign({}, nextConfig, {
            exportPathMap: async (defaultPathMap, { dir }) => {
                log_1.default('Validating Firebase Hosting Rules...');
                const firebaseConfigPath = await find_up_1.default(FirebaseConfigFileName, {
                    cwd: dir,
                });
                if (!firebaseConfigPath) {
                    throw new Error(`${FirebaseConfigFileName} not found looking through directories from ${dir} to the root.`);
                }
                log_1.default(`Found Firebase config at ${firebaseConfigPath}.`);
                const firebaseConfig = JSON.parse(await promises_1.default.readFile(firebaseConfigPath, 'utf8'));
                const firebaseConfigHosting = firebaseConfig.hosting;
                if (!firebaseConfigHosting) {
                    throw new Error(`The config ${firebaseConfigPath} does not contain a "hosting" section. Add it and run this again.`);
                }
                const firebaseConfigHostingItems = firebaseConfigHosting.filter((hostingConfig) => hostingConfig.site && sites.includes(hostingConfig.site));
                if (firebaseConfigHostingItems.length === 0) {
                    throw new Error(`The following sites were provided as parameter (${sites.join(', ')}), but no hosting entries with matching site were found.`);
                }
                if (firebaseConfigHostingItems.length !== sites.length) {
                    const foundSites = firebaseConfigHostingItems.map((item) => item.site);
                    throw new Error(`The following sites were provided as parameter (${sites.join(', ')}), but only (${foundSites.join(', ')}).`);
                }
                let errorFound = false;
                for (const sitePath in defaultPathMap) {
                    if (sitePath.includes('[')) {
                        // Replace slugs (i.e. [...varname])
                        let source = sitePath.replace(/\[\.{3}.+?\]$/gm, '**');
                        // Check for mistake in remaining params (param starting with .)
                        if (/\[\.+.+?\]/gm.test(source)) {
                            throw new Error(`Dynamic path '${sitePath}' section should not start with . (e.g. [.name]) except for slugs at the end (i.e. [...name]). Check https://nextjs.org/docs/routing/dynamic-routes for more details.`);
                        }
                        // Replace other params with glob
                        source = source.replace(/\[[^.].+?\]/gm, '*');
                        // Destination just add html (generated by Next.js)
                        let destination = sitePath + '.html';
                        let rewriteRule;
                        for (const hostingEntry of firebaseConfigHostingItems) {
                            rewriteRule = hostingEntry.rewrites?.find((val) => val.source === source);
                            if (!rewriteRule) {
                                errorFound = true;
                                console.error(`\n[ERROR] @ firebase.json [hosting/site=${hostingEntry.site}]\nMissing rewrites rule for path '${sitePath}'. Include the following rewrite rule in firebase.json.\n`);
                                console.error(JSON.stringify({ source, destination }));
                                console.error('\n');
                            }
                            else if (rewriteRule.destination !== destination) {
                                errorFound = true;
                                console.error(`\n[ERROR] @ firebase.json [hosting/site=${hostingEntry.site}]\nIncorrect rewrites rule for path '${sitePath}'. In firebase.json replace:\n`);
                                console.error(JSON.stringify(rewriteRule));
                                console.error('\nwith\n');
                                console.error(JSON.stringify({ source, destination }));
                                console.error('');
                            }
                        }
                    }
                }
                if (errorFound) {
                    throw new Error(`One or more rewrites rules missing or erroneous in '${firebaseConfigPath}'.`);
                }
                else {
                    log_1.default('[PASS] Firebase Hosting Rules are OK!');
                }
                return defaultPathMap;
            },
        });
    };
//# sourceMappingURL=index.js.map